import PropTypes from 'prop-types';
export var types = ['array', 'bool', 'func', 'number', 'object', 'string', 'symbol', 'node', 'element', 'elementType', 'instanceOf', 'oneOf', 'oneOfType', 'arrayOf', 'objectOf', 'shape', 'exact', 'any'];
export var factory = function factory() {
  var T = {
    checkPropTypes: checkPropTypes
  };

  var _loop = function _loop(i) {
    var name = types[i];
    var propType = PropTypes[name];
    var hasArg = propType.isRequired === undefined;

    if (!hasArg) {
      T[name] = {
        name: name,
        propType: propType
      };
      T[name].isRequired = {
        name: name,
        propType: propType.isRequired
      };
    } else {
      T[name] = function (arg) {
        var result = propType(extractPropTypes(arg));
        var typedef = {
          name: name,
          arg: arg,
          propType: result
        };
        typedef.isRequired = {
          name: name,
          arg: arg,
          propType: result.isRequired
        };
        return typedef;
      };
    }
  };

  for (var i = 0; i < types.length; i++) {
    _loop(i);
  }

  return T;
};
export var checkPropTypes = function checkPropTypes(schema) {
  var typeSpecs = extractPropTypes(schema);

  for (var _len = arguments.length, rest = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    rest[_key - 1] = arguments[_key];
  }

  return PropTypes.checkPropTypes.apply(PropTypes, [typeSpecs].concat(rest));
};
export var extractPropTypes = function extractPropTypes(arg) {
  var extract = function extract(val) {
    return val && val.propType || val;
  };

  var isTypeObject = arg.propType && arg.name;

  if (typeof arg === 'object') {
    if (arg instanceof Array) {
      return arg.map(extract);
    } else if (isTypeObject) {
      return extract(arg);
    } else {
      var res = {};

      for (var key in arg) {
        res[key] = extract(arg[key]);
      }

      return res;
    }
  } else {
    return arg;
  }
};
export var T = factory();
export default T;