"use strict";

exports.__esModule = true;
exports["default"] = exports.initialStateFromSchema = exports.initStruct = exports.initProp = exports.initializers = void 0;

var _react = require("react");

var initializers = {
  array: function array() {
    return [];
  },
  bool: function bool() {
    return false;
  },
  func: function func() {
    return function () {};
  },
  number: function number() {
    return 0;
  },
  object: function object() {
    return {};
  },
  string: function string() {
    return '';
  },
  symbol: function symbol() {
    return Symbol();
  },
  node: function node() {
    return '';
  },
  element: function element() {
    return /*#__PURE__*/(0, _react.createElement)('div');
  },
  elementType: function elementType() {
    return function () {
      return /*#__PURE__*/(0, _react.createElement)('div');
    };
  },
  instanceOf: function instanceOf(arg) {
    return new arg();
  },
  oneOf: function oneOf(arg) {
    return arg[0];
  },
  oneOfType: function oneOfType(arg) {
    return initProp(arg[0], true);
  },
  arrayOf: function arrayOf() {
    return [];
  },
  objectOf: function objectOf() {
    return {};
  },
  shape: function shape(arg) {
    return initStruct(arg);
  },
  exact: function exact(arg) {
    return initStruct(arg);
  },
  any: function any() {
    return '';
  }
};
exports.initializers = initializers;

var initProp = function initProp(type, force) {
  if (!type || !type.name) throw new Error('Invalid entity schema'); // A bit confusing but a required type is NOT expected to have an `isRequired` option

  var required = type.isRequired === undefined;
  var init = initializers[type.name];
  if (typeof init !== 'function') throw new Error('Invalid entity schema');
  return required || force ? init(type.arg) : null;
};

exports.initProp = initProp;

var initStruct = function initStruct(struct) {
  var o = {};

  for (var key in struct) {
    o[key] = initProp(struct[key]);
  }

  return o;
};

exports.initStruct = initStruct;

var initialStateFromSchema = function initialStateFromSchema(schema) {
  return initStruct(schema);
};

exports.initialStateFromSchema = initialStateFromSchema;
var _default = initialStateFromSchema;
exports["default"] = _default;